<?php
/*-----------------------------------------------------------------------------
    Photo Visual Confirmation - A phpBB Add-On
  ----------------------------------------------------------------------------
    photo.php
       CAPTCHA Image Display File
    File Version: 1.1.0
    Begun: May 29, 2007                 Last Modified: June 4, 2007
  ----------------------------------------------------------------------------
    Copyright 2007 by Jeremy Rogers.  Please read the license.txt included
    with the phpBB Add-On listed above for full license and copyright details.
  ----------------------------------------------------------------------------
    Class and Function Quick Reference
             Names                                 Search Labels
         captcha........................................[capt]
            execute.....................................[exec]
            show_preview................................[shwprv]
            show_images.................................[shwimg]
-----------------------------------------------------------------------------*/

/*	captcha                                                 [capt]
       Display class for Photo Visual Confirmation.
*/
class captcha
{
	var $width	= 112;
	var $height	= 84;

/*	execute                                                 [exec]
       Displays a photo image for Photo Visual Confirmation.  Each image has
	   random gamma and size transformations before being displayed.

    Arguments:
       $code	- The confirmation code for which an image is being displayed.
       $seed	- A random seed.  Don't use this!  It breaks the gamma and
					cropping transformations on the images.

    Return:
       None.
*/
	function execute($code, $seed)
	{
		global $config, $phpbb_root_path;

		if ( @!extension_loaded('gd') )
		{
			// No GD? Let's stop now.
			return FALSE;
		}

		$idx = request_var('idx', 0);
		$code = substr(base_convert($code, 16, 2), $idx, 1);

		// Init the random generator.
		// You may be tempted to use $seed in this. Don't. It breaks the
		// image adjustments performed later.
		mt_srand();

		if ($code == '0')
		{
			$directory = $phpbb_root_path . $config['photo_captcha_fail_path'];
		}
		else
		{
			$directory = $phpbb_root_path . $config['photo_captcha_pass_path'];
		}
		if( substr($directory, -1) != '/' )
		{
			$directory .= '/';
		}

		// Get images
		$options = array();
		$dirlist = opendir($directory);
		while( ($jpg = readdir($dirlist)) !== false)
		{
			if(!is_dir($jpg))
			{
				if (strtolower(substr($jpg, -4, 4)) == '.jpg')
				{
					$options[] = $jpg;
				}
			}
		}
		closedir($dirlist);
		if (empty($options))
		{
			exit;
		}

		$selindex = intval(base_convert(substr(gen_rand_string(12), 0, 7), 16, 10));
		$selindex = $selindex % count($options);
		$fullpath = $directory . $options[$selindex];

		$image = @imagecreatefromjpeg($fullpath);
		if (!$image)
		{
			exit;
		}

		// Create the transformed image
		$targetx = @imagesx($image);
		$targety = @imagesy($image);
		$sourcex = mt_rand(($targetx * 7)/10, ($targetx * 9)/10);
		$sourcey = mt_rand(($targety * 7)/10, ($targety * 9)/10);
		$offx = mt_rand(0, $targetx - $sourcex);
		$offy = mt_rand(0, $targety - $sourcey);
		$sub = imagecreatetruecolor($this->width, $this->height);
		@imagecopyresampled($sub, $image, 0, 0, $offx, $offy, $this->width, $this->height, $sourcex, $sourcey);
		$image = $sub;
		@imagegammacorrect($image, 1.0, (0.5 + mt_rand(0,1200)*0.001));

		// Print the image
		header('Content-Type: image/jpeg');
		header('Cache-control: no-cache, no-store');
		@imagejpeg($image);
		imagedestroy($image);
	}

/*	show_preview                                            [shwprv]
       Displays a preview of the photo captcha in the admin panel.

    Arguments:
       $url_append	- A string to be added to the end of image URLs.
       $id			- Indentifier for the preview mode.

    Return:
       string  $confirm_image  The HTML for the captcha display.
*/
	function show_preview($url_append, $id)
	{
		global $phpbb_admin_path, $phpEx, $config;
		
		$num_rows = $config['photo_captcha_' . CONFIRM_REG . '_rows'];
		$num_columns = $config['photo_captcha_' . CONFIRM_REG . '_number'];
		$num_pics = $num_columns * $num_rows;

		$confirm_image = '<table>';
		for ($tablerow=0; $tablerow<$num_rows; $tablerow++) {
			$confirm_image .= '<tr>';
			for ($tablecol=0; $tablecol<$num_columns; $tablecol++) {
				$idx = $tablerow*$num_columns + $tablecol;
				$confirm_image .= '<td><img src="' . append_sid("{$phpbb_admin_path}index.$phpEx", "i=$id&amp;demo=demo&amp;idx=$idx" . $url_append) . '" alt="" title="" width="' . $this->width . '" height="' . $this->height . '" /></td>';
				$confirm_image .= '<td><input type="checkbox" name="captchabox' . $idx . '" /></td>';
				$confirm_image .= '<td width="50"></td>';
			}
			$confirm_image .= '</tr>';
		}
		$confirm_image .= '</table>';

		return $confirm_image;
	}

//       $code			- The confirmation code being used.  This is for
//							debugging only!

/*	show_images                                             [shwimg]
       Displays the photo captcha.

    Arguments:
       $type		- Type of confirm code. This should be passed as a constant
						defined in constants.php.
       $confirm_id	- ID of the confirmation code stored in the database.
       $url_append	- A string to be added to the end of image URLs.

    Return:
       string  $confirm_image  The HTML for the captcha display.
*/
	function show_images($type, $confirm_id, $url_append = '')
	{
		global $phpbb_root_path, $phpEx, $config, $template, $user;

		$template->assign_var('L_CONFIRM_CODE_EXPLAIN', sprintf($user->lang['CONFIRM_CODE_EXPLAIN'], '<a href="mailto:' . htmlspecialchars($config['board_contact']) . '">', '</a>'));

		$template->set_filenames(array(
			'photo_confirm'	=> 'mods/photo_confirm.html'
		));

		$num_rows = $config['photo_captcha_' . $type . '_rows'];
		$num_columns = $config['photo_captcha_' . $type . '_number'];
		$num_pics = $num_columns * $num_rows;

		for ($tablerow=0; $tablerow<$num_rows; $tablerow++)
		{
			$template->assign_block_vars('photo_row', array());
			for ($tablecol=0; $tablecol<$num_columns; $tablecol++)
			{
				$idx = $tablerow*$num_columns + $tablecol;
				$template->assign_block_vars('photo_row.photo_col', array(
					'IMG_URL'	=> append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=confirm&amp;id=' . $confirm_id . '&amp;type=' . $type . "&amp;idx=$idx" . $url_append),
					'IDX'		=> $idx,
					'WIDTH'		=> $this->width,
					'HEIGHT'	=> $this->height,
				));
			}
		}
		return $template->assign_display('photo_confirm', '', true);
	}
}

?>