<?php
/*-----------------------------------------------------------------------------
    Photo Visual Confirmation - A phpBB Add-On
  ----------------------------------------------------------------------------
    functions.php
       Common Operations File
    File Version: 1.0.0
    Begun: May 29, 2007                 Last Modified: May 30, 2007
  ----------------------------------------------------------------------------
    Copyright 2007 by Jeremy Rogers.  Please read the license.txt included
    with the phpBB Add-On listed above for full license and copyright details.
  ----------------------------------------------------------------------------
    Class and Function Quick Reference
             Names                                 Search Labels
         generate_code................................[gencde]
         convert_code.................................[crtcde]
         show_confirm_image...........................[shwimg]
         request_confirm_code.........................[reqcde]
-----------------------------------------------------------------------------*/

/*	generate_code                                           [gencde]
       Generates a confirmation code.  Replaces the phpBB 3 default method of
	   generating codes.  That method creates codes of variable length, which
	   breaks Photo Visual Confirmation.

    Arguments:
       None.

    Return:
       string  The confirmation code.
*/
function generate_code()
{
	global $config;
	if( !$config['photo_captcha'] )
	{
		return gen_rand_string(mt_rand(5, 8));
	}
	return substr(unique_id(), 0, 6);
}

/*	convert_code                                            [crtcde]
       Converts a confirmation code pulled from the database to the format used
	   by Photo Visual Confirmation.
	   If Photo Visual Confirmation is disabled, the confirm code is returned
	   with no changes.

    Arguments:
       $confirm_code	- A confirmation code pulled from the database.
       $type			- A CONFIRM_ constant defined in constants.php.
							This is used to get the settings for the specific
							type of captcha.

    Return:
       string  The confirmation code.
*/
function convert_code($confirm_code, $type)
{
	global $config;
	if( !$config['photo_captcha'] )
	{
		return $confirm_code;
	}
	$num_rows = $config['photo_captcha_' . $type . '_rows'];
	$num_columns = $config['photo_captcha_' . $type . '_number'];
	$num_pics = $num_columns * $num_rows;
	return substr(base_convert($confirm_code, 16, 2), 0, $num_pics);
}

/*	show_confirm_image                                      [shwimg]
       Builds a string of HTML code needed to display a confirmation image
	   or images.

    Arguments:
       $type			- A CONFIRM_ constant defined in constants.php.
							This is used to get the settings for the specific
							type of captcha.
       $confirm_id		- The confirm_id of a confirm code in the database.
       $url_append		- A string added to the end of the URL of the image(s).
       $code			- The confirmation code being used.  This is for
							debugging only!

    Return:
       string  The HTML for the confirmation image(s).
*/
function show_confirm_image($type, $confirm_id, $url_append = '', $code = '')
{
	global $phpbb_root_path, $phpEx, $config;

	if( !$config['photo_captcha'] )
	{
		// Not using photo captcha - return a default image.
		return '<img src="' . append_sid("{$phpbb_root_path}ucp.$phpEx", 'mode=confirm&amp;id=' . $confirm_id . '&amp;type=' . $type . $url_append) . '" alt="" title="" />';
	}

	// We're using the photo captcha. Let's build the display.
	include($phpbb_root_path . 'includes/captcha/photo.' . $phpEx);
	$captcha = new captcha();
	return $captcha->show_images($type, $confirm_id, $url_append, $code);
}

/*	request_confirm_code                                    [reqcde]
       Retrieves a confirmation code from passed form input.  This is basically
	   a complex wrapper for phpBB 3's request_var().  Since each photo captcha
	   may have a different number of checkboxes, we use this to combine the
	   checkboxes into one code.

    Arguments:
       $type			- A CONFIRM_ constant defined in constants.php.
							This is used to get the settings for the specific
							type of captcha.

    Return:
       string  The complete confirmation code.
*/
function request_confirm_code($type)
{
	global $config;
	if( !$config['photo_captcha'] )
	{
		// Photo Captcha disabled - return the basic code.
		return request_var('confirm_code', '');
	}

	// Run through the whole set for this type
	// of confirmation to build the code.
	$confirm_code = '';
	$num_rows = $config['photo_captcha_' . $type . '_rows'];
	$num_columns = $config['photo_captcha_' . $type . '_number'];
	$num = $num_columns * $num_rows;
	for ($box=0; $box < $num; $box++)
	{
		if (request_var("captchabox$box", '') == 'on')
		{
			$confirm_code .= '1';
		}
		else
		{
			$confirm_code .= '0';
		}
	}
	return $confirm_code;
}

?>